@@ -1,7 +1,11 @@
+#ifndef SQUID_CONFIG_H
+#include "config.h"
+#endif
+
 #ifndef SQUID_OS_HPUX_H
 #define SQUID_OS_PHUX_H
 
-#if _SQUID_HPUX_
+#ifdef _SQUID_HPUX_
 
 /****************************************************************************
  *--------------------------------------------------------------------------*
@@ -23,13 +27,6 @@
 #define getrusage(a, b)  syscall(SYS_GETRUSAGE, a, b)
 #endif
 
-/*
- * getpagesize(...) not available on HPUX
- */
-#if !defined(HAVE_GETPAGESIZE)
-#define HAVE_GETPAGESIZE
-#define getpagesize( )   sysconf(_SC_PAGE_SIZE)
-#endif
 
 #endif /* _SQUID_HPUX_ */
 #endif /* SQUID_OS_HPUX_H */
